@@ -32,21 +32,34 @@
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
-#include <bluetooth/hci_lib.h>
-#include <bluetooth/rfcomm.h>
-#include <bluetooth/l2cap.h>
-#include <bluetooth/sco.h>
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
 
 #include <glib.h>
 
+#include "btio.h"
+#include "gattrib.h"
+#include "att.h"
+#include "gatt.h"
+#include "sdpd.h"
 #include "glib-helper.h"
 
 /* Number of seconds to keep a sdp_session_t in the cache */
 #define CACHE_TIMEOUT 2
 
+struct gattrib_context {
+	bdaddr_t src;
+	bdaddr_t dst;
+	GAttrib *attrib;
+	GIOChannel *io;
+	bt_primary_t cb;
+	bt_destroy_t destroy;
+	gpointer user_data;
+	GSList *primaries;
+};
+
+static GSList *gattrib_list = NULL;
+
 struct cached_sdp_session {
 	bdaddr_t src;
 	bdaddr_t dst;
@@ -56,6 +69,22 @@ struct cached_sdp_session {
 
 static GSList *cached_sdp_sessions = NULL;
 
+static void gattrib_context_free(struct gattrib_context *ctxt)
+{
+	gattrib_list = g_slist_remove(gattrib_list, ctxt);
+	if (ctxt->destroy)
+		ctxt->destroy(ctxt->user_data);
+
+	g_slist_free(ctxt->primaries);
+	g_attrib_unref(ctxt->attrib);
+	if (ctxt->io) {
+		g_io_channel_unref(ctxt->io);
+		g_io_channel_shutdown(ctxt->io, FALSE, NULL);
+	}
+
+	g_free(ctxt);
+}
+
 static gboolean cached_session_expired(gpointer user_data)
 {
 	struct cached_sdp_session *cached = user_data;
@@ -94,7 +123,7 @@ static sdp_session_t *get_sdp_session(const bdaddr_t *src, const bdaddr_t *dst)
 }
 
 static void cache_sdp_session(bdaddr_t *src, bdaddr_t *dst,
-				sdp_session_t *session)
+						sdp_session_t *session)
 {
 	struct cached_sdp_session *cached;
 
@@ -188,8 +217,8 @@ done:
 	search_context_cleanup(ctxt);
 }
 
-static gboolean search_process_cb(GIOChannel *chan,
-			GIOCondition cond, void *user_data)
+static gboolean search_process_cb(GIOChannel *chan, GIOCondition cond,
+							gpointer user_data)
 {
 	struct search_context *ctxt = user_data;
 	int err = 0;
@@ -218,7 +247,8 @@ failed:
 	return FALSE;
 }
 
-static gboolean connect_watch(GIOChannel *chan, GIOCondition cond, gpointer user_data)
+static gboolean connect_watch(GIOChannel *chan, GIOCondition cond,
+							gpointer user_data)
 {
 	struct search_context *ctxt = user_data;
 	sdp_list_t *search, *attrids;
@@ -275,8 +305,9 @@ failed:
 }
 
 static int create_search_context(struct search_context **ctxt,
-				const bdaddr_t *src, const bdaddr_t *dst,
-				uuid_t *uuid)
+					const bdaddr_t *src,
+					const bdaddr_t *dst,
+					uuid_t *uuid)
 {
 	sdp_session_t *s;
 	GIOChannel *chan;
@@ -331,17 +362,7 @@ int bt_search_service(const bdaddr_t *src, const bdaddr_t *dst,
 	return 0;
 }
 
-int bt_discover_services(const bdaddr_t *src, const bdaddr_t *dst,
-		bt_callback_t cb, void *user_data, bt_destroy_t destroy)
-{
-	uuid_t uuid;
-
-	sdp_uuid16_create(&uuid, PUBLIC_BROWSE_GROUP);
-
-	return bt_search_service(src, dst, &uuid, cb, user_data, destroy);
-}
-
-static int find_by_bdaddr(const void *data, const void *user_data)
+static gint find_by_bdaddr(gconstpointer data, gconstpointer user_data)
 {
 	const struct search_context *ctxt = data, *search = user_data;
 
@@ -349,21 +370,16 @@ static int find_by_bdaddr(const void *data, const void *user_data)
 					bacmp(&ctxt->src, &search->src));
 }
 
-int bt_cancel_discovery(const bdaddr_t *src, const bdaddr_t *dst)
+static gint gattrib_find_by_bdaddr(gconstpointer data, gconstpointer user_data)
 {
-	struct search_context search, *ctxt;
-	GSList *match;
+	const struct gattrib_context *ctxt = data, *search = user_data;
 
-	memset(&search, 0, sizeof(search));
-	bacpy(&search.src, src);
-	bacpy(&search.dst, dst);
+	return (bacmp(&ctxt->dst, &search->dst) &&
+					bacmp(&ctxt->src, &search->src));
+}
 
-	/* Ongoing SDP Discovery */
-	match = g_slist_find_custom(context_list, &search, find_by_bdaddr);
-	if (!match)
-		return -ENODATA;
-
-	ctxt = match->data;
+static int cancel_sdp(struct search_context *ctxt)
+{
 	if (!ctxt->session)
 		return -ENOTCONN;
 
@@ -374,6 +390,179 @@ int bt_cancel_discovery(const bdaddr_t *src, const bdaddr_t *dst)
 		sdp_close(ctxt->session);
 
 	search_context_cleanup(ctxt);
+
+	return 0;
+}
+
+static int cancel_gattrib(struct gattrib_context *ctxt)
+{
+	if (ctxt->attrib)
+		g_attrib_cancel_all(ctxt->attrib);
+
+	gattrib_context_free(ctxt);
+
+	return 0;
+}
+
+int bt_cancel_discovery(const bdaddr_t *src, const bdaddr_t *dst)
+{
+	struct search_context sdp_ctxt;
+	struct gattrib_context gatt_ctxt;
+	GSList *match;
+
+	memset(&sdp_ctxt, 0, sizeof(sdp_ctxt));
+	bacpy(&sdp_ctxt.src, src);
+	bacpy(&sdp_ctxt.dst, dst);
+
+	/* Ongoing SDP Discovery */
+	match = g_slist_find_custom(context_list, &sdp_ctxt, find_by_bdaddr);
+	if (match)
+		return cancel_sdp(match->data);
+
+	memset(&gatt_ctxt, 0, sizeof(gatt_ctxt));
+	bacpy(&gatt_ctxt.src, src);
+	bacpy(&gatt_ctxt.dst, dst);
+
+	/* Ongoing Discover All Primary Services */
+	match = g_slist_find_custom(gattrib_list, &gatt_ctxt,
+						gattrib_find_by_bdaddr);
+	if (match == NULL)
+		return -ENOTCONN;
+
+	return cancel_gattrib(match->data);
+}
+
+static void primary_cb(guint8 status, const guint8 *pdu, guint16 plen,
+							gpointer user_data)
+{
+	struct gattrib_context *ctxt = user_data;
+	struct att_data_list *list;
+	unsigned int i, err;
+	uint16_t start, end;
+
+	if (status == ATT_ECODE_ATTR_NOT_FOUND) {
+		err = 0;
+		goto done;
+	}
+
+	if (status != 0) {
+		err = -EIO;
+		goto done;
+	}
+
+	list = dec_read_by_grp_resp(pdu, plen);
+	if (list == NULL) {
+		err = -EPROTO;
+		goto done;
+	}
+
+	for (i = 0, end = 0; i < list->num; i++) {
+		const uint8_t *data = list->data[i];
+		struct att_primary *primary;
+		uuid_t u128, u16;
+
+		start = att_get_u16(&data[0]);
+		end = att_get_u16(&data[2]);
+
+		if (list->len == 6) {
+			sdp_uuid16_create(&u16,
+					att_get_u16(&data[4]));
+			sdp_uuid16_to_uuid128(&u128, &u16);
+
+		} else if (list->len == 20)
+			sdp_uuid128_create(&u128, &data[4]);
+		else
+			/* Skipping invalid data */
+			continue;
+
+		primary = g_try_new0(struct att_primary, 1);
+		if (!primary) {
+			err = -ENOMEM;
+			goto done;
+		}
+		primary->start = start;
+		primary->end = end;
+		sdp_uuid2strn(&u128, primary->uuid, sizeof(primary->uuid));
+		ctxt->primaries = g_slist_append(ctxt->primaries, primary);
+	}
+
+	att_data_list_free(list);
+	err = 0;
+
+	if (end != 0xffff) {
+		gatt_discover_primary(ctxt->attrib, end + 1, 0xffff, NULL,
+							primary_cb, ctxt);
+		return;
+	}
+
+done:
+	ctxt->cb(ctxt->primaries, err, ctxt->user_data);
+	gattrib_context_free(ctxt);
+}
+
+static void connect_cb(GIOChannel *io, GError *gerr, gpointer user_data)
+{
+	struct gattrib_context *ctxt = user_data;
+
+	if (gerr) {
+		ctxt->cb(NULL, -EIO, ctxt->user_data);
+		gattrib_context_free(ctxt);
+		return;
+	}
+
+	ctxt->attrib = g_attrib_new(io);
+	gatt_discover_primary(ctxt->attrib, 0x0001, 0xffff, NULL, primary_cb,
+									ctxt);
+}
+
+int bt_discover_primary(const bdaddr_t *src, const bdaddr_t *dst, int psm,
+					bt_primary_t cb, void *user_data,
+					gboolean secure,
+					bt_destroy_t destroy)
+{
+	struct gattrib_context *ctxt;
+	BtIOSecLevel sec_level;
+	GIOChannel *io;
+
+	ctxt = g_try_new0(struct gattrib_context, 1);
+	if (ctxt == NULL)
+		return -ENOMEM;
+
+	bacpy(&ctxt->src, src);
+	bacpy(&ctxt->dst, dst);
+	ctxt->user_data = user_data;
+	ctxt->cb = cb;
+	ctxt->destroy = destroy;
+
+	if (secure == TRUE)
+		sec_level = BT_IO_SEC_HIGH;
+	else
+		sec_level = BT_IO_SEC_LOW;
+
+	if (psm < 0)
+		io = bt_io_connect(BT_IO_L2CAP, connect_cb, ctxt, NULL, NULL,
+				BT_IO_OPT_SOURCE_BDADDR, src,
+				BT_IO_OPT_DEST_BDADDR, dst,
+				BT_IO_OPT_CID, GATT_CID,
+				BT_IO_OPT_SEC_LEVEL, sec_level,
+				BT_IO_OPT_INVALID);
+	else
+		io = bt_io_connect(BT_IO_L2CAP, connect_cb, ctxt, NULL, NULL,
+				BT_IO_OPT_SOURCE_BDADDR, src,
+				BT_IO_OPT_DEST_BDADDR, dst,
+				BT_IO_OPT_PSM, psm,
+				BT_IO_OPT_SEC_LEVEL, sec_level,
+				BT_IO_OPT_INVALID);
+
+	if (io == NULL) {
+		gattrib_context_free(ctxt);
+		return -EIO;
+	}
+
+	ctxt->io = io;
+
+	gattrib_list = g_slist_append(gattrib_list, ctxt);
+
 	return 0;
 }
 
@@ -445,7 +634,7 @@ static struct {
 	{ }
 };
 
-uint16_t bt_name2class(const char *pattern)
+static uint16_t name2class(const char *pattern)
 {
 	int i;
 
@@ -466,6 +655,24 @@ static inline gboolean is_uuid128(const char *string)
 			string[23] == '-');
 }
 
+static int string2uuid16(uuid_t *uuid, const char *string)
+{
+	int length = strlen(string);
+	char *endptr = NULL;
+	uint16_t u16;
+
+	if (length != 4 && length != 6)
+		return -EINVAL;
+
+	u16 = strtol(string, &endptr, 16);
+	if (endptr && *endptr == '\0') {
+		sdp_uuid16_create(uuid, u16);
+		return 0;
+	}
+
+	return -EINVAL;
+}
+
 char *bt_name2string(const char *pattern)
 {
 	uuid_t uuid;
@@ -477,7 +684,7 @@ char *bt_name2string(const char *pattern)
 		return g_strdup(pattern);
 
 	/* Friendly service name format */
-	uuid16 = bt_name2class(pattern);
+	uuid16 = name2class(pattern);
 	if (uuid16)
 		goto proceed;
 
@@ -524,14 +731,14 @@ int bt_string2uuid(uuid_t *uuid, const char *string)
 
 		return 0;
 	} else {
-		uint16_t class = bt_name2class(string);
+		uint16_t class = name2class(string);
 		if (class) {
 			sdp_uuid16_create(uuid, class);
 			return 0;
 		}
-	}
 
-	return -1;
+		return string2uuid16(uuid, string);
+	}
 }
 
 gchar *bt_list2string(GSList *list)
